/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.client;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ibv.uda.uda.data.Skript;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * die Repräsentation eines lokalen Uda-Skripts innerhalb einer
 * Client-Anwendung.<br>
 * Ein lokales Skript hat keine Beziehung zu einem UdaModul. Der zugeordnete
 * Skripttext liegt in einer lokalen Datei vor, die beim Abfragen des
 * Quelltextes eingelesen wird. Ein derartiges Skriptobjekt, kann einem
 * LokalenSkriptlauf zugeordnet und lokal ausgeführt werden, d.h. ohne die
 * "Umfassende Datenanalyse" einzubeziehen.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class LokalesSkript extends Skript {

	private static final Debug LOGGER = Debug.getLogger();
	/**
	 * die Datei, die dem Quelltext des Skripts enthält.
	 */
	private InputStream skriptInput;

	/**
	 * Konstruktor.<br>
	 * Die Funktion erzeugt ein lokales Skriptobjekt, dessen Quelltext in der
	 * Datei mit dem übergebenen Namen abgelegt ist.
	 *
	 * @param fileName
	 *            der Name der Datei mit dem Quelltext des Skripts
	 */
	public LokalesSkript(final String fileName) {
		super(new File(fileName).getName());
		try {
			skriptInput = new FileInputStream(new File(fileName));
		} catch (final FileNotFoundException e) {
			LokalesSkript.LOGGER.error(e.getLocalizedMessage());
			// der zur Verfügung gestellte Sourcecode ist leer
		}
	}

	/**
	 * Konstruktor.<br>
	 * Die Funktion erzeugt ein lokales Skriptobjekt, dessen Quelltext in dem
	 * übergebenen Stream bereitgestellt wird.
	 *
	 * @param name
	 *            Name des Skripts
	 * @param stream
	 *            der Quelltext des Skripts als Stream
	 */
	public LokalesSkript(final String name, final InputStream stream) {
		super(name);
		skriptInput = stream;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Da ein lokales Skriptobjekt kein zugeordnetes Datenverteilerobjekt
	 * besitzt, wird in jedem Fall eine {@link UnsupportedOperationException}
	 * geworfen.
	 *
	 * @see de.bsvrz.ibv.uda.uda.data.Skript#getDavObjekt()
	 */
	@Override
	public SystemObject getDavObjekt() {
		throw new UnsupportedOperationException("Ein lokales Skript hat kein zugeordnetes Datenverteilerobjekt");
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion überlädt, die entsprechende Funktion der Superklasse. Beim
	 * Abfragen des Sourcecodes wird immer zuerst die dem Skript zugeordnete
	 * Datei eingelesen und der innerhalb des Skriptobjekts verwaltete
	 * Sourcecode aktualisiert.
	 *
	 * @see de.bsvrz.ibv.uda.uda.data.Skript#getSourceCode()
	 */
	@Override
	public List<String> getSourceCode() {
		clearSourceCode();
		try {
			final BufferedReader reader = new BufferedReader(new InputStreamReader(skriptInput));
			String line;
			while ((line = reader.readLine()) != null) {
				addSourceLine(line);
			}
		} catch (final IOException e) {
			LokalesSkript.LOGGER.error(e.getLocalizedMessage());
			// Sourcecode bleibt leer, wenn die Datei nicht eingelesen werden
			// konnte
		}
		return super.getSourceCode();
	}
}
