/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.dav;

import java.io.IOException;
import java.util.List;

import com.bitctrl.Constants;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.archive.ArchiveData;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKindCombination;
import de.bsvrz.dav.daf.main.archive.ArchiveDataQueryResult;
import de.bsvrz.dav.daf.main.archive.ArchiveDataSpecification;
import de.bsvrz.dav.daf.main.archive.ArchiveDataStream;
import de.bsvrz.dav.daf.main.archive.ArchiveOrder;
import de.bsvrz.dav.daf.main.archive.ArchiveQueryPriority;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestManager;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestOption;
import de.bsvrz.dav.daf.main.archive.ArchiveTimeSpecification;
import de.bsvrz.dav.daf.main.archive.TimingType;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ibv.uda.interpreter.AbstractUdaHandler;
import de.bsvrz.ibv.uda.interpreter.daten.container.Feld;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.Fehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.KonfigurationsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.ZeitBereich;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.Zeitstempel;
import de.bsvrz.ibv.uda.uda.UdaMessage;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.bitctrl.interpreter.HandlerValidation;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Handler zur Ausführung von Operationen, die sich auf Elemente und Daten aus
 * dem Datenverteiler beziehen.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class DavHandler extends AbstractUdaHandler {

	/**
	 * Logger für Debugausgaben.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * ermittelt Archivdaten für die übergebene Datenspezifikation.
	 *
	 * @param objekt
	 *            das Systemobjekt
	 * @param atg
	 *            dte Attributgruppe
	 * @param aspekt
	 *            der Aspekt
	 * @param bereich
	 *            der Zeitraum
	 * @param nurGeandert
	 *            nur geänderte Daten lesen
	 * @param nachgelieferte
	 *            auch nachgelieferte Daten ermitteln
	 * @return ein Feld mit den gelesenen Daten
	 */
	private Feld archivDatenLesen(final SystemObjectObjekt objekt, final AttributeGroup atg, final Aspect aspekt,
			final ZeitBereich bereich, final boolean nurGeandert, final boolean nachgelieferte) {
		final ArchiveRequestManager archiv = objekt.getVerbindung().getArchive();
		final Feld result = new Feld();
		if (archiv.isArchiveAvailable()) {

			ArchiveRequestOption requestOption = ArchiveRequestOption.NORMAL;
			if (nurGeandert) {
				requestOption = ArchiveRequestOption.DELTA;
			}

			final DataDescription dataDesc = new DataDescription(atg, aspekt);
			final ArchiveTimeSpecification timeSpec = new ArchiveTimeSpecification(TimingType.DATA_TIME, false,
					bereich.getStart().getMilliSekunden(), bereich.getEnde().getMilliSekunden());
			ArchiveDataKindCombination dataKinds = null;
			if (nachgelieferte) {
				dataKinds = new ArchiveDataKindCombination(ArchiveDataKind.ONLINE, ArchiveDataKind.ONLINE_DELAYED);
			} else {
				dataKinds = new ArchiveDataKindCombination(ArchiveDataKind.ONLINE);
			}
			final ArchiveDataSpecification spec = new ArchiveDataSpecification(timeSpec, dataKinds,
					ArchiveOrder.BY_DATA_TIME, requestOption, dataDesc, objekt.getObjekt());

			// Anfrage verschicken und auf Antwort warten
			final ArchiveDataQueryResult antwort = archiv.request(ArchiveQueryPriority.MEDIUM, spec);

			try {
				assert antwort.getStreams().length == 1;
				final ArchiveDataStream strom = antwort.getStreams()[0];

				// Daten auslesen und in Feld eintragen
				while (true) {
					final ArchiveData archivdaten;
					final Data daten;
					final long timestamp;

					archivdaten = strom.take();
					if (archivdaten == null) {
						// Datenstrom ist zu Ende
						break;
					}

					timestamp = archivdaten.getDataTime();
					daten = archivdaten.getData();
					if (daten == null) {
						continue; // ein leerer Datensatz wird ignoriert
					}

					result.anhaengen(0, DavDaten.erzeugeObjekt(objekt.getVerbindung(),
							new ResultData(objekt.getObjekt(), dataDesc, timestamp, daten)));
				}
			} catch (final IllegalStateException e) {
				LogTools.log(DavHandler.LOGGER, UdaMessage.ErrorArchivAnfrage, e.getMessage());
			} catch (final InterruptedException e) {
				LogTools.log(DavHandler.LOGGER, UdaMessage.ErrorArchivAnfrage, e.getMessage());
			} catch (final IOException e) {
				LogTools.log(DavHandler.LOGGER, UdaMessage.ErrorArchivAnfrage, e.getMessage());
			}
		}

		return result;
	}

	/**
	 * liest die Daten einer Attributgruppe eines Systemobjekts aus.
	 *
	 * @param operanden
	 *            die Operanden für die Ausführung der Operation
	 * @return die Daten
	 */
	private Object datenLesen(final List<Object> operanden) {
		Object result = null;
		if (operanden.size() >= 3) {
			final SystemObjectObjekt objekt = (SystemObjectObjekt) operanden.get(0);
			final ClientDavInterface connection = objekt.getVerbindung();
			final AttributeGroup atg;
			final Aspect aspekt;
			Object operand = operanden.get(1);
			if (operand instanceof String) {
				atg = connection.getDataModel().getAttributeGroup((String) operand);
			} else if (operand instanceof AtgObjekt) {
				atg = ((AtgObjekt) operand).getObjekt();
			} else {
				throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
			}

			if (atg == null) {
				throw new KonfigurationsFehler(UdaFehlerSubtyp.ATTRIBUTGRUPPE);
			}

			operand = operanden.get(2);
			if (operand instanceof String) {
				aspekt = connection.getDataModel().getAspect((String) operand);
			} else if (operand instanceof AspektObjekt) {
				aspekt = ((AspektObjekt) operand).getObjekt();
			} else {
				throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
			}

			if (aspekt == null) {
				throw new KonfigurationsFehler(UdaFehlerSubtyp.ASPEKT);
			}

			if (operanden.size() >= 4) {
				boolean nachgelieferte = false;
				boolean geaenderte = false;

				final ZeitBereich bereich = (ZeitBereich) operanden.get(3);
				if (operanden.size() > 4) {
					geaenderte = ((LogischerWert) operanden.get(4)).getBoolWert();
				}
				if (operanden.size() > 5) {
					nachgelieferte = ((LogischerWert) operanden.get(5)).getBoolWert();
				}
				result = archivDatenLesen(objekt, atg, aspekt, bereich, geaenderte, nachgelieferte);
			} else {
				ResultData data = null;
				if (atg.getAttributeGroupUsage(aspekt).isConfigurating()) {
					data = new ResultData(objekt.getObjekt(), new DataDescription(atg, aspekt), 0,
							(objekt.getObjekt()).getConfigurationData(atg));
				} else {
					data = connection.getData(objekt.getObjekt(), new DataDescription(atg, aspekt), 0);
				}

				result = DavDaten.erzeugeObjekt(connection, data);
			}

			if (result == null) {
				result = NichtWert.NICHTWERT;
			}
			return result;
		}

		throw new UnsupportedOperationException();
	}

	/**
	 * schreibt die Daten für eine Attributgruppe eines Systemobjekts.
	 *
	 * @param operanden
	 *            die Operanden der Operation
	 */
	private void datenSchreiben(final List<Object> operanden) {
		if (operanden.size() == 5) {
			final SystemObjectObjekt objekt = (SystemObjectObjekt) operanden.get(0);
			final ClientDavInterface connection = objekt.getVerbindung();
			final Zeitstempel zs = (Zeitstempel) operanden.get(1);
			final AttributeGroup atg;
			final Aspect aspekt;
			Object operand = operanden.get(2);
			if (operand instanceof String) {
				atg = connection.getDataModel().getAttributeGroup((String) operand);
			} else if (operand instanceof AtgObjekt) {
				atg = ((AtgObjekt) operand).getObjekt();
			} else {
				throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
			}

			if (atg == null) {
				throw new KonfigurationsFehler(UdaFehlerSubtyp.ATTRIBUTGRUPPE);
			}

			operand = operanden.get(3);
			if (operand instanceof String) {
				aspekt = connection.getDataModel().getAspect((String) operand);
			} else if (operand instanceof AspektObjekt) {
				aspekt = ((AspektObjekt) operand).getObjekt();
			} else {
				throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
			}

			if (aspekt == null) {
				throw new KonfigurationsFehler(UdaFehlerSubtyp.ASPEKT);
			}

			final DavDatenObjekt daten = (DavDatenObjekt) operanden.get(4);
			final DataDescription dataDesc = new DataDescription(atg, aspekt);
			final BestaetigterDavSender sender = BestaetigterDavSender.getSender(connection);

			boolean angemeldet = false;

			try {
				if (!sender.isQuelle(objekt.getObjekt(), dataDesc)) {
					sender.anmelden(objekt.getObjekt(), dataDesc, false);
					angemeldet = true;
					sender.warteAufBestaetigung(objekt.getObjekt(), dataDesc, Constants.MILLIS_PER_MINUTE);
				}
			} catch (@SuppressWarnings("unused") final SendSubscriptionNotConfirmed e) {
				throw new Fehler("Sendeanmeldung nicht innerhalb der angegebenen Zeit erfolgt");
			}
			try {
				connection.sendData(
						new ResultData(objekt.getObjekt(), dataDesc, zs.getMilliSekunden(), daten.getObjekt()));
			} catch (final DataNotSubscribedException e) {
				throw new Fehler(e.getMessage());
			} catch (final SendSubscriptionNotConfirmed e) {
				throw new Fehler(e.getMessage());
			} catch (final IllegalArgumentException e) {
				throw new Fehler(e.getMessage());
			}
			if (angemeldet) {
				sender.abmelden(objekt.getObjekt(), dataDesc);
			}
		} else {
			throw new UnsupportedOperationException();
		}
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @return die Liste der unterstützten Operatoren
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#getHandledOperators()
	 */
	@Override
	public Operator[] getHandledOperators() {
		return DavOperatoren.OPERATOREN;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#perform(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public Object perform(final Operator operator, final List<Object> operanden) {

		erzeugeHandlerFehler(operator, operanden);

		Object result = null;

		try {
			if (DavOperatoren.DATEN_LESEN.equals(operator)) {
				result = datenLesen(operanden);
			} else if (DavOperatoren.LOESCHEN.equals(operator)) {
				final DynamischesObjekt objekt = (DynamischesObjekt) operanden.get(0);
				try {
					((SystemObject) objekt.getObjekt()).invalidate();
					result = objekt;
				} catch (@SuppressWarnings("unused") final ConfigurationChangeException e) {
					throw new KonfigurationsFehler(UdaFehlerSubtyp.UNBEKANNT);
				}
			} else if (DavOperatoren.DATENSATZ.equals(operator)) {
				final AtgObjekt atgObjekt = (AtgObjekt) operanden.get(0);
				final AttributeGroup atg = atgObjekt.getObjekt();
				result = DavDaten.erzeugeObjekt(atgObjekt.getVerbindung(), atgObjekt.getVerbindung().createData(atg));
			} else if (DavOperatoren.DATEN_SCHREIBEN.equals(operator)) {
				datenSchreiben(operanden);
				result = NichtWert.NICHTWERT;
			} else if (DavOperatoren.ENTFERNEN.equals(operator)) {
				final MengenObjekt menge = (MengenObjekt) operanden.get(0);
				result = operanden.get(1);
				menge.entferneObjekt((SystemObjectObjekt) result);
			} else if (DavOperatoren.EINFUEGEN.equals(operator)) {
				final MengenObjekt menge = (MengenObjekt) operanden.get(0);
				result = operanden.get(1);
				menge.einfuegenObjekt((SystemObjectObjekt) result);
			} else if (DavOperatoren.DATA_TYP.equals(operator)) {
				final DavDatenObjekt daten = (DavDatenObjekt) operanden.get(0);
				final Data data = daten.getObjekt();
				result = DavDaten.erzeugeObjekt(daten.getVerbindung(), data.getAttributeType());
			} else if (DavOperatoren.DATA_NAME.equals(operator)) {
				result = ((DavDatenObjekt) operanden.get(0)).getObjekt().getName();
			} else if (DavOperatoren.DATA_EINHEIT.equals(operator)) {
				result = ((DavDatenObjekt) operanden.get(0)).getEinheit();
			} else if (DavOperatoren.DATA_INFO.equals(operator)) {
				result = ((DavDatenObjekt) operanden.get(0)).getInfo();
			} else if (DavOperatoren.DATA_WERTEBEREICH.equals(operator)) {
				final Object operand = operanden.get(0);
				if (operand instanceof DavDatenObjekt) {
					result = ((DavDatenObjekt) operanden.get(0)).getWertebereich();
				} else if (operand instanceof AttributeTypeObjekt) {
					result = ((AttributeTypeObjekt) operanden.get(0)).getStrukturElement(operator.getSymbol());
				}
			} else if (DavOperatoren.DATA_STATUSLISTE.equals(operator)) {
				final Object operand = operanden.get(0);
				if (operand instanceof DavDatenObjekt) {
					result = ((DavDatenObjekt) operanden.get(0)).getStatusListe();
				} else if (operand instanceof AttributeTypeObjekt) {
					result = ((AttributeTypeObjekt) operanden.get(0)).getStrukturElement(operator.getSymbol());
				}
			} else if (DavOperatoren.DATA_WERT.equals(operator)) {
				result = ((DavDatenObjekt) operanden.get(0)).getWertObjekt();
			}
		} catch (@SuppressWarnings("unused") final ClassCastException e) {
			throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
		}
		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#validiereHandler(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public HandlerValidation validiereHandler(final Operator operator, final List<? extends Object> operanden) {
		boolean anzahlOk = false;
		boolean typOk = false;
		if (DavOperatoren.DATEN_LESEN.equals(operator)) {
			if (operanden.size() >= 3) {
				anzahlOk = true;
				if (isValidOperand(operanden, 0, SystemObjectObjekt.class)) {
					if (isValidOperand(operanden, 1, String.class, AtgObjekt.class)) {
						if (isValidOperand(operanden, 2, String.class, AspektObjekt.class)) {
							switch (operanden.size()) {
							case 3:
								typOk = true;
								break;
							case 4:
								if (operanden.get(3) instanceof ZeitBereich) {
									typOk = true;
								}
								break;
							case 5:
								if ((operanden.get(3) instanceof ZeitBereich)
										&& (operanden.get(4) instanceof LogischerWert)) {
									typOk = true;
								}
								break;
							case 6:
								if ((operanden.get(3) instanceof ZeitBereich)
										&& (operanden.get(4) instanceof LogischerWert)
										&& (operanden.get(5) instanceof LogischerWert)) {
									typOk = true;
								}
								break;
							default:
								break;
							}
						}
					}
				}
			}
		} else if (DavOperatoren.DATEN_SCHREIBEN.equals(operator)) {
			if (operanden.size() == 5) {
				anzahlOk = true;
				if (operanden.get(0) instanceof SystemObjectObjekt) {
					if (operanden.get(1) instanceof Zeitstempel) {
						final Object atgObjekt = operanden.get(2);
						if ((atgObjekt instanceof String) || (atgObjekt instanceof AtgObjekt)) {
							final Object aspObjekt = operanden.get(3);
							if ((aspObjekt instanceof String) || (aspObjekt instanceof AspektObjekt)) {
								if (operanden.get(4) instanceof DavDatenObjekt) {
									typOk = true;
								}
							}
						}
					}
				}
			}
		} else if (DavOperatoren.DATENSATZ.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				if (operanden.get(0) instanceof AtgObjekt) {
					typOk = true;
				}
			}
		} else if (DavOperatoren.LOESCHEN.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				if (operanden.get(0) instanceof DynamischesObjekt) {
					typOk = true;
				}
			}
		} else if (DavOperatoren.EINFUEGEN.equals(operator)) {
			if (operanden.size() == 2) {
				anzahlOk = true;
				if ((operanden.get(0) instanceof MengenObjekt) && (operanden.get(1) instanceof DynamischesObjekt)) {
					typOk = true;
				}
			}
		} else if (DavOperatoren.ENTFERNEN.equals(operator)) {
			if (operanden.size() == 2) {
				anzahlOk = true;
				if ((operanden.get(0) instanceof MengenObjekt) && (operanden.get(1) instanceof DynamischesObjekt)) {
					typOk = true;
				}
			}
		} else if (DavOperatoren.DATA_TYP.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				if (operanden.get(0) instanceof DavDatenObjekt) {
					typOk = true;
				}
			}
		} else if (DavOperatoren.DATA_NAME.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				if (operanden.get(0) instanceof DavDatenObjekt) {
					typOk = true;
				}
			}
		} else if (DavOperatoren.DATA_EINHEIT.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				if (operanden.get(0) instanceof DavDatenObjekt) {
					typOk = true;
				}
			}
		} else if (DavOperatoren.DATA_WERTEBEREICH.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				if (operanden.get(0) instanceof DavDatenObjekt) {
					typOk = true;
				} else if (operanden.get(0) instanceof AttributeTypeObjekt) {
					typOk = true;
				}
			}
		} else if (DavOperatoren.DATA_STATUSLISTE.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				if (operanden.get(0) instanceof DavDatenObjekt) {
					typOk = true;
				} else if (operanden.get(0) instanceof AttributeTypeObjekt) {
					typOk = true;
				}
			}
		} else if (DavOperatoren.DATA_INFO.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				if (operanden.get(0) instanceof DavDatenObjekt) {
					typOk = true;
				}
			}
		} else if (DavOperatoren.DATA_WERT.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				if (operanden.get(0) instanceof DavDatenObjekt) {
					typOk = true;
				}
			}
		}

		return new HandlerValidation(anzahlOk, typOk);
	}
}
