/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.konstante;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.daten.zeit.ZeitDauer;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Literal;

/**
 * Die Repräsentation eines Literals, das den Wert einer relativen Zeitangabe
 * innerhalb des Uda-Interpreterbaumes repräsentiert. Eine Zeitangabe setzt sich
 * zusammen aus einer Menge von Paaren von Wert und Zeiteinheit, z.B.:
 *
 * <pre>
 *        &lt;b&gt;1 Stunde 5 Minuten 3 Sekunden&lt;/b&gt;
 * </pre>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class RelativeZeitLiteral implements Literal {

	/**
	 * die Einheiten für eine Zeitdauer.
	 */
	public enum ZeitEinheit {
		/**
		 * Angabe in Tagen.
		 */
		TAG, /**
		 * Angabe in Stunden.
		 */
		STUNDE, /**
		 * Angabe in Minuten.
		 */
		MINUTE, /**
		 * Angabe in Sekunden.
		 */
		SEKUNDE, /**
		 * Angabe in Milisekunden.
		 */
		MILLISEKUNDE
	}

	/**
	 * Interne Klasse, die einen Teil einer relativen Zeitangabe repräsentiert.
	 *
	 * @author Peuker
	 */
	private static class ZeitAngabe implements Literal {
		/**
		 * der Wert der Zeitangabe.
		 */
		private final ZahlenLiteral value;

		/**
		 * der Zeiteinheit.
		 */
		private final ZeitEinheit einheit;

		/**
		 * Konstruktor mit Angabe der Zeiteinheit und des Wertes.
		 *
		 * @param value
		 *            der Wert
		 * @param einheit
		 *            die Zeiteinheit
		 */
		ZeitAngabe(final ZahlenLiteral value, final ZeitEinheit einheit) {
			this.value = value;
			this.einheit = einheit;
		}

		/**
		 * liefert den Wert der Teilangabe in Millisekunden.
		 *
		 * @param kontext
		 *            der Kontext, in dem der Ausdruck ausgewertet wird
		 * @return den Wert für die Zeitdauer
		 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
		 */
		@Override
		public Number interpret(final Kontext kontext) {
			double zeitInMsec = value.interpret(kontext).doubleValue();
			switch (einheit) {
			case TAG:
				zeitInMsec *= 24L * 60L * 60L * 1000L;
				break;
			case STUNDE:
				zeitInMsec *= 60L * 60L * 1000L;
				break;
			case MINUTE:
				zeitInMsec *= 60L * 1000L;
				break;
			case SEKUNDE:
				zeitInMsec *= 1000L;
				break;
			case MILLISEKUNDE:
				break;
			default:
				// andere Zeitbestandteile werden hier nicht erwartet
				break;
			}
			return zeitInMsec;
		}

		/**
		 * {@inheritDoc}.
		 *
		 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
		 */
		@Override
		public List<Ausdruck> getNachfolger() {
			return null;
		}

		/**
		 * {@inheritDoc}.
		 *
		 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Literal#getWert()
		 */
		@Override
		public Number getWert() {
			return interpret(null);
		}
	}

	/**
	 * die Liste der Zeitangaben, aus denen der Ausdruck zusammengesetzt ist.
	 */
	private final Collection<ZeitAngabe> zeiten = new ArrayList<>();

	/**
	 * Die Auswertung des Ausdrucks liefert in jedem Fall eine Instanz der
	 * Klasse <i>Zeitdauer</i>.
	 *
	 * @param kontext
	 *            der Kontext, in dem der Ausdruck ausgewertet wird
	 * @return die Zeitdauer, die durch den Ausdruck repräsentiert wird
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public final ZeitDauer interpret(final Kontext kontext) {
		long milliSekunden = 0L;
		for (final ZeitAngabe angabe : zeiten) {
			milliSekunden += angabe.interpret(kontext).longValue();
		}
		return new ZeitDauer(milliSekunden);
	}

	/**
	 * die Funktion fügt dem Ausdruck eine weitere Zeitangabe, d.h. ein paar von
	 * Wert und Faktor(Zeiteinheit) hinzu.
	 *
	 * @param value
	 *            der Wert
	 * @param einheit
	 *            die Zeiteinheit
	 */
	public void addZeitAngabe(final ZahlenLiteral value, final ZeitEinheit einheit) {
		zeiten.add(new ZeitAngabe(value, einheit));
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Literal#getWert()
	 */
	@Override
	public ZeitDauer getWert() {
		return interpret(null);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		return new ArrayList<>(zeiten);
	}
}
