/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.struktur;

import de.bsvrz.ibv.uda.interpreter.daten.container.Container;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ContainerZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;

/**
 * Die Klasse beschreibt ein Objekt zur Repräsentation einer benutzerdefinierten
 * Datenstrukturdefinition.<br>
 * Die Klasse implementiert die Schnittstelle Container, um den indizierten
 * Zugriff auf die Elemente der Struktur zu ermöglichen.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class DatenStrukturDefinition implements Container {

	/**
	 * die Namen der Elemente der Datenstruktur.
	 */
	private final String[] fields;

	/**
	 * Konstruktor mit Angabe der Attributnamen der Datenstruktur.
	 *
	 * @param fields
	 *            die Namen der Datenstruktur-Attribute
	 */
	public DatenStrukturDefinition(final String[] fields) {
		this.fields = fields;
		if (this.fields != null) {
			for (int idx = 0; idx < this.fields.length; idx++) {
				this.fields[idx] = this.fields[idx].trim();
			}
		}
		validiereFelder();
	}

	/**
	 * Die Funktion &uuml;berpr&uuml;ft die G&uuml;ltigkeit der Feldnamen.<br>
	 * Es muss mindestens ein Feld definiert sein. Alle Felder müssen
	 * g&uuml;ltige Namen haben.
	 */
	private void validiereFelder() {
		if ((fields == null) || (fields.length == 0)) {
			throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE,
					"Eine Datenstruktur muss mindestens ein Attribut enthalten");
		}
		for (final String feld : fields) {
			if (!isValidAttributName(feld)) {
				throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE, "Der Attributname \""
						+ feld + "\" ist für eine Datenstruktur nicht zulässig");
			}
		}
		for (int idx = 0; idx < fields.length; idx++) {
			for (int check = idx + 1; check < fields.length; check++) {
				if (fields[idx].equals(fields[check])) {
					throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE,
							"Attributname doppelt verwendet: " + fields[idx]);
				}
			}
		}

	}

	/**
	 * Die Funktion pr&uuml;ft die G&uuml;ltigkeit eines Feldnamens.<br>
	 * Feldnamen können mit einem Buchstaben, dem Zeichen '_' oder einem Umlaut
	 * beginnen. Die folgenden Zeichen entsprechen dem Anfangszeichen,
	 * zus&auml;tzlich ist die Verwendung von Ziffern erlaubt.
	 *
	 * @param feld
	 *            der zu prüfende Name
	 * @return die G&uuml;ltigkeit
	 */
	private boolean isValidAttributName(final String feld) {
		return feld.matches("[A-Za-z[äüöÄÜÖß_]][A-Za-z[äüöÄÜÖß_[0-9]]]*?");
	}

	/**
	 * liefert eine Zeichenkette, die die Datenstruktur repräsentiert.<br>
	 * Die Ausgabe erfolgt in der Form
	 * "{&lt;attribut1&gt;,&lt;attribut2&gt;,...}" {@inheritDoc}.
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		final StringBuffer buffer = new StringBuffer("{");
		for (final String field : fields) {
			if (buffer.length() > 1) {
				buffer.append(",");
			}
			buffer.append(field);
		}
		buffer.append('}');
		return buffer.toString();
	}

	/**
	 * liefert die Liste der Attributnamen der Datenstrukturdefinition.
	 *
	 * @return die Liste der Attributnamen
	 */
	public String[] getFields() {
		return fields;
	}

	/**
	 * Implementierung der entsprechenden Funktion der Schnittstelle
	 * <i>Container</i>.<br>
	 * Die Funktion liefert den Namen des über den Index beschriebenen
	 * Attributs. Das erste Element hat den Index 1. Wird ein ungültiger Index
	 * angegeben, wird ein <i>ObjektZugriffsFehler</i> geworfen.
	 *
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.Container#getContainerElement(int)
	 * @see #setContainerElement(int, Object)
	 */
	@Override
	public Object getContainerElement(final int index) {
		Object result = null;
		try {
			result = fields[index - 1];
		} catch (@SuppressWarnings("unused") final IndexOutOfBoundsException e) {
			result = new ContainerZugriffsFehler(UdaFehlerSubtyp.INDEX,
					"Zugriff auf Element " + index + " nicht möglich");
		}
		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.Container#setContainerElement(int,
	 *      java.lang.Object)
	 * @see #getContainerElement(int)
	 */
	@Override
	public void setContainerElement(final int index, final Object daten) {
		throw new ArgumentFehler(UdaFehlerSubtyp.METHODE);
	}
}
