/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.uda.data;

import java.util.ArrayList;
import java.util.List;

import com.bitctrl.Constants;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ibv.uda.uda.UdaKonstante;

/**
 * die Repräsentation der Daten eines UdaSkripts.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public abstract class Skript {

	/**
	 * der Name des Skripts.
	 */
	private String name = UdaKonstante.STRING_UNBEKANNT;

	/**
	 * die Beschreibung des Skriptinhalts.
	 */
	private String beschreibung = Constants.EMPTY_STRING;

	/**
	 * das Erstellungsdatum.
	 */
	private long erstellungsDatum = System.currentTimeMillis();

	/**
	 * der Name des Nutzers, der das Skript erstellt hat.
	 */
	private String ersteller = UdaKonstante.STRING_UNBEKANNT;

	/**
	 * die Historie für die Bearbeitung des Skripts.
	 */
	private final List<Historie> historie = new ArrayList<>();

	/**
	 * der Zustand der Skriptbearbeitung.
	 */
	private SkriptZustand zustand = SkriptZustand.ANGELEGT;

	/**
	 * der Sourcecode des Skripts.
	 */
	private final List<String> sourceCode = new ArrayList<>();

	/**
	 * Konstruktor.<br>
	 * Die Funktion erzeugt ein Skriptobjekt mit dem übergebenen Namen.
	 *
	 * @param name
	 *            der Name
	 */
	public Skript(final String name) {
		if ((name != null) && (name.trim().length() > 0)) {
			this.name = name.trim();
		}
	}

	/**
	 * fügt dem Skript einen Historie-Eintrag hinzu.
	 *
	 * @param eintrag
	 *            der neue Eintrag
	 */
	public void addHistorie(final Historie eintrag) {
		historie.add(eintrag);
	}

	/**
	 * fügt dem Quellcode eine neue Zeile hinzu.
	 *
	 * @param line
	 *            die neue Zeile
	 */
	public void addSourceLine(final String line) {
		sourceCode.add(line);
	}

	/**
	 * löscht die Liste der Historie-Einträge.
	 */
	public void clearHistorie() {
		historie.clear();
	}

	/**
	 * löscht alle Zeilen des Quellcodes.
	 */
	public void clearSourceCode() {
		sourceCode.clear();
	}

	/**
	 * liefert die erklärende Beschreibung des Skriptinhalts.
	 *
	 * @return die Beschreibung
	 */
	public String getBeschreibung() {
		return beschreibung;
	}

	/**
	 * liefert des Datenverteiler-Systemobjekt, mit dem das Skript innerhalb des
	 * Datenverteilers repräsentiert wird.
	 *
	 * @return das Systemobjekt
	 */
	public abstract SystemObject getDavObjekt();

	/**
	 * liefert den Namen des Erstellers.
	 *
	 * @return den Namen
	 */
	public String getErsteller() {
		return ersteller;
	}

	/**
	 * liefert das Erstellungsdatum des Skripts in Millisekunden seit dem
	 * 1.1.1970 0 Uhr GMT.
	 *
	 * @return den Erstellungszeitpunkt
	 */
	public long getErstellungsDatum() {
		return erstellungsDatum;
	}

	/**
	 * liefert die Liste der Einträge der Bearbeitungsgeschichte des Skripts.
	 *
	 * @return die Einträge
	 */
	public List<Historie> getHistorie() {
		return historie;
	}

	/**
	 * liefert den Namen des Skripts.
	 *
	 * @return den Namen
	 */
	public String getName() {
		return name;
	}

	/**
	 * liefert den Quellcode des Skripts als Zeichenkette.
	 *
	 * @return der Quelltext
	 */
	public String getSource() {
		final StringBuffer source = new StringBuffer();
		for (final String line : getSourceCode()) {
			source.append(line);
			source.append(Constants.NL);
		}
		return source.toString();
	}

	/**
	 * liefert den Quellcode als Liste von Zeilen.
	 *
	 * @return den Quellcode
	 */
	public List<String> getSourceCode() {
		return sourceCode;
	}

	/**
	 * liefert den Bearbeitungsszustand des Skripts.
	 *
	 * @return den Zustand
	 * @see #setZustand(SkriptZustand)
	 */
	public SkriptZustand getZustand() {
		return zustand;
	}

	/**
	 * ermittelt, ob das Skript für die Ausführung freigegeben wurde.
	 *
	 * @return <i>true</i>, wenn eine Freigabe erfolgt ist.
	 */
	public boolean isFreigegeben() {
		return SkriptZustand.FREIGEGEBEN.equals(zustand);
	}

	/**
	 * übernimmt die Daten des übergebenen Skript.
	 *
	 * @param skript
	 *            das Skript, dessen Daten übernommen werden sollen
	 */
	public void kopiereDaten(final Skript skript) {
		if (skript != null) {
			setSkriptName(skript.getName());
			setSkriptBeschreibung(skript.getBeschreibung());
			setErstelltDurch(skript.getErsteller());
			setErstelltAm(skript.getErstellungsDatum());
			clearHistorie();
			for (final Historie eintrag : skript.getHistorie()) {
				addHistorie(eintrag);
			}
			clearSourceCode();
			for (final String eintrag : skript.getSourceCode()) {
				addSourceLine(eintrag);
			}
			setZustand(skript.getZustand());
		}
	}

	/**
	 * setzt das Erstellungsdatum des Skripts auf den in Millisekunden seit dem
	 * 1.1.1970 0 Uhr GMT übergebenen Zeitpunkt.
	 *
	 * @param erstelltAm
	 *            der Zeitpunkt
	 */
	public void setErstelltAm(final long erstelltAm) {
		this.erstellungsDatum = erstelltAm;
	}

	/**
	 * setzt den Namen des Ersteööers des Skripts.
	 *
	 * @param erstelltDurch
	 *            der Name
	 */
	public void setErstelltDurch(final String erstelltDurch) {
		this.ersteller = erstelltDurch;
	}

	/**
	 * setzt den erklärenden Text des Skripts.
	 *
	 * @param skriptBeschreibung
	 *            der beschreibende Text
	 */
	public void setSkriptBeschreibung(final String skriptBeschreibung) {
		this.beschreibung = skriptBeschreibung;
	}

	/**
	 * setzt den Namen des Skripts.
	 *
	 * @param skriptName
	 *            der Name
	 */
	public void setSkriptName(final String skriptName) {
		if ((skriptName != null) && (skriptName.trim().length() > 0)) {
			this.name = skriptName.trim();
		}
	}

	/**
	 * setzt den Bearbeitungszustand des Skripts.
	 *
	 * @param zustand
	 *            der Zustand
	 * @see #getZustand()
	 */
	public void setZustand(final SkriptZustand zustand) {
		this.zustand = zustand;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Es wird der den Name des Skripts ausgegeben.
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return getName();
	}
}
