/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.uda.data;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.bitctrl.Constants;

import de.bsvrz.ibv.uda.interpreter.UdaInterpreter;
import de.bsvrz.ibv.uda.uda.UdaKonstante;
import de.bsvrz.ibv.uda.verwaltung.protokoll.Protokoll;
import de.bsvrz.ibv.uda.verwaltung.protokoll.ProtokollEintrag;

/**
 * Allgemeine Repräsentation der Daten eines Skriptlaufs.<br>
 * Ein Skriptlauf beischreibt die Ausführungsumgebung für ein zugeordnetes
 * Uda-Skript und die Bedingungen für seine Ausführung.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public abstract class SkriptLauf {

	/**
	 * das zugeordnete Skript.
	 */
	private Skript skript;

	/**
	 * der Name des Skriptlaufs.
	 */
	private String name;

	/**
	 * eine ausführlichere Beschreibung des Inhalts des Skriptlaufs.
	 */
	private String beschreibung;

	/**
	 * die Liste der Parameter des Skriptlaufs.
	 */
	private final Map<String, String> skriptParameter = new HashMap<>();

	/**
	 * der Name des Benutzers, unter dem das Skript innerhalb der Umfassenden
	 * Datenanalyse ausgeführt werden soll.
	 */
	private String benutzer;

	/**
	 * das Passwort des Benutzers.
	 */
	private String passwd;

	/**
	 * der Starttyp für die Ausführung des Skripts.
	 */
	private StartTyp startTyp = StartTyp.EXPLIZIT;

	/**
	 * der Zyklus für eine potentielle zyklische Ausführung.
	 */
	private final StartZyklus zyklus = new StartZyklus(0, 0);

	/**
	 * die Liste der Protokolle für die Ausführung des Skripts.
	 */
	private final Collection<Protokoll> protokolle = new ArrayList<>();

	/**
	 * die Historie für die Bearbeitung des Skriptlaufs.
	 */
	private final List<Historie> historie = new ArrayList<>();

	/**
	 * der Status der Ausführung des Skripts.
	 */
	private SkriptLaufStatus status = SkriptLaufStatus.PASSIV;

	/**
	 * der Interpreter, der den Skriptlauf ausführt.
	 */
	private UdaInterpreter interpreter;

	/**
	 * fügt dem Skript einen Historie-Eintrag hinzu.
	 *
	 * @param eintrag
	 *            der neue Eintrag
	 */
	public void addHistorie(final Historie eintrag) {
		historie.add(eintrag);
	}

	/**
	 * fügt dem Skriptlauf eine Protokolldefinition hinzu.
	 *
	 * @param protokoll
	 *            das Protokoll
	 */
	public void addProtokoll(final Protokoll protokoll) {
		protokolle.add(protokoll);
	}

	/**
	 * gibt die übergebenen Daten über alle dem Skriptlauf zugewiesenen
	 * Ausgabakanäle aus.
	 *
	 * @param daten
	 *            die auszugebenden Daten
	 */
	public void ausgeben(final ProtokollEintrag daten) {
		for (final Protokoll protokoll : getProtokolle()) {
			protokoll.ausgeben(daten);
		}
	}

	/**
	 * löscht die Liste der Historie-Einträge.
	 */
	public void clearHistorie() {
		historie.clear();
	}

	/**
	 * löscht die Liste der Parameter des Skripts.
	 */
	public void clearParameter() {
		skriptParameter.clear();
	}

	/**
	 * entfernt alle für den Skriptlauf definierten Protokolleinträge.
	 */
	public void clearProtokolle() {
		protokolle.clear();
	}

	/**
	 * liefert den Name des Benutzers mit dem die Anmeldung beim Datenverteiler
	 * bei der Asuführung des Skripts erfolgen soll.
	 *
	 * @return der Name
	 * @see #setBenutzer(String)
	 */
	public String getBenutzer() {
		if (benutzer == null) {
			benutzer = "Tester";
		}
		return benutzer;
	}

	/**
	 * liefert die erläuternde Beschreibung des Skriptlaufs.
	 *
	 * @return die Beschreibung
	 * @see #setBeschreibung(String)
	 */
	public String getBeschreibung() {
		if (beschreibung == null) {
			beschreibung = Constants.EMPTY_STRING;
		}
		return beschreibung;
	}

	/**
	 * liefert die Liste der Einträge der Bearbeitungsgeschichte des Skripts.
	 *
	 * @return die Einträge
	 */
	public List<Historie> getHistorie() {
		return historie;
	}

	/**
	 * liefert den ausführenden Interpreter.
	 *
	 * @return den Interpreter
	 * @see #setInterpreter(UdaInterpreter)
	 */
	public UdaInterpreter getInterpreter() {
		return interpreter;
	}

	/**
	 * liefert das Modul, in dem der Skriptlauf definiert ist.
	 *
	 * @return das Modul
	 */
	public abstract Modul getModul();

	/**
	 * liefert den Namen des Skripts.<br>
	 * Ist kein Name definiert wird die Zeichenkette
	 * {@link UdaKonstante#STRING_UNBEKANNT} zurückgeliefert.
	 *
	 * @return den Name oder {@link UdaKonstante#STRING_UNBEKANNT}
	 * @see #setName(String)
	 */
	public String getName() {
		if ((name == null) || (name.length() <= 0)) {
			name = UdaKonstante.STRING_UNBEKANNT;
		}
		return name;
	}

	/**
	 * liefert die Liste der Parameter, mit denen das dem Skriptlauf zugeordnete
	 * Skript ausgeführt werden soll.
	 *
	 * @return parameter die Liste der Parameter
	 * @see #setParameter(String, String)
	 */
	public Map<String, String> getParameter() {
		return skriptParameter;
	}

	/**
	 * liefert des Passwort düe die Anmeldung beim Datenverteiler bei der
	 * Ausführung des Skriptlaufs.
	 *
	 * @return das Passwort
	 * @see #setPasswd(String)
	 */
	public String getPasswd() {
		if (passwd == null) {
			passwd = Constants.EMPTY_STRING;
		}
		return passwd;
	}

	/**
	 * liefert die Liste der Protokolle, die bei der Ausführung des Skripts
	 * erstellt werden sollen.
	 *
	 * @return protokoll die Liste der Protokolle
	 */
	public Collection<Protokoll> getProtokolle() {
		return protokolle;
	}

	/**
	 * liefert das dem Skriptlauf zugeordnete Skript.
	 *
	 * @return das Skript oder <i>null</i>, wenn kein Skript zugeordnet wurde.
	 * @see #setSkript(Skript)
	 */
	public Skript getSkript() {
		return skript;
	}

	/**
	 * liefert die Liste der für das Skript definierten Parameter.
	 *
	 * @return die Liste der Parameter
	 */
	public Map<String, String> getSkriptParameter() {
		return skriptParameter;
	}

	/**
	 * liefert den für das Skript definierten Starttyp.
	 *
	 * @return den Starttyp
	 * @see #setStartTyp(StartTyp)
	 */
	public StartTyp getStartTyp() {
		return startTyp;
	}

	/**
	 * liefert den aktuellen Status der Ausführung des Skripts.
	 *
	 * @return den Status
	 * @see #setStatus(SkriptLaufStatus)
	 */
	public SkriptLaufStatus getStatus() {
		return status;
	}

	/**
	 * liefert die Daten für eine zyklische oder zeitgesteuerte Ausführung des
	 * Skriptlaufs.
	 *
	 * @return zyklus die Zyklusbeschreibung
	 * @see #setZyklus(StartZyklus)
	 */
	public StartZyklus getZyklus() {
		return zyklus;
	}

	/**
	 * initialisiert alle Protokolle vor der Ausführung des zugeordneten
	 * Skripts.
	 */
	public void initialisiereProtokolle() {
		for (final Protokoll protokoll : getProtokolle()) {
			protokoll.initialisiere(this);
		}
	}

	/**
	 * die Funktion übernimmt die Daten des übergebenen Skriptlaufs.
	 *
	 * @param quellLauf
	 *            der Skriptlauf, von dem die Daten übernommen werden sollen.
	 */
	public void kopiereDaten(final SkriptLauf quellLauf) {
		if (quellLauf != null) {
			setBenutzer(quellLauf.getBenutzer());
			setBeschreibung(quellLauf.getBeschreibung());
			clearParameter();
			for (final String key : quellLauf.getParameter().keySet()) {
				setParameter(key, quellLauf.getParameter().get(key));
			}
			setName(quellLauf.getName());
			setPasswd(quellLauf.getPasswd());
			setSkript(quellLauf.getSkript());
			setStartTyp(quellLauf.getStartTyp());
			setZyklus(quellLauf.getZyklus());

			clearProtokolle();
			for (final Protokoll protokoll : quellLauf.getProtokolle()) {
				addProtokoll(protokoll);
			}
		}
	}

	/**
	 * schließt alle Protokolle des Skriptlaufs.
	 */
	public void protokolleAbschliessen() {
		for (final Protokoll protokoll : getProtokolle()) {
			protokoll.abschliessen();
		}
	}

	/**
	 * veröffentlicht das Ergebnis eines Skriptlaufs.
	 *
	 * @param fehler
	 *            die Ursache eines Skriptabbruchs oder <i>null</i>, wenn das
	 *            Skript ordnungsgemäß beendet wurde.
	 */
	public abstract void publiziereErgebnis(Exception fehler);

	/**
	 * entfernt den mit dem Namen benannten Parameter des Skripts.
	 *
	 * @param parameter
	 *            der Parametername
	 */
	public void removeParameter(final String parameter) {
		skriptParameter.remove(parameter);
	}

	/**
	 * entfernt einen Protokolleintrag aus den Definitionen des Skriptlaufs.
	 *
	 * @param item
	 *            die Protokolldefinition
	 */
	public void removeProtokoll(final Protokoll item) {
		protokolle.remove(item);
	}

	/**
	 * setzt den Namen des Benutzers, unter dessen Namen das Skript ausgeführt
	 * werden soll.
	 *
	 * @param benutzer
	 *            der Name des Benutzers
	 * @see #getBenutzer()
	 */
	public void setBenutzer(final String benutzer) {
		this.benutzer = benutzer;
	}

	/**
	 * setzt den erläuternden Text des Skriptlaufs.
	 *
	 * @param beschreibung
	 *            der beschreibende Text
	 * @see #getBeschreibung()
	 */
	public void setBeschreibung(final String beschreibung) {
		this.beschreibung = beschreibung;
	}

	/**
	 * setzt den Interpreter, der den Skriptlauf ausführt.
	 *
	 * @param interpreter
	 *            der Interpreter
	 * @see #getInterpreter()
	 */
	public void setInterpreter(final UdaInterpreter interpreter) {
		this.interpreter = interpreter;
	}

	/**
	 * setzt den Namen des Skriptlaufs.
	 *
	 * @param name
	 *            der Name
	 * @see #getName()
	 */
	public void setName(final String name) {
		this.name = name;
	}

	/**
	 * setzt einen Startparameter des Skripts.<br>
	 * Die Namen für einen Parameter sind eindeutig, d.h. beim Aufruf eines
	 * Skripts kann der gleiche Parameter nicht mehrfach verwendet werden.
	 *
	 * @param parameter
	 *            der Parametername
	 * @param wert
	 *            der Wert
	 * @see #getParameter()
	 */
	public void setParameter(final String parameter, final String wert) {
		skriptParameter.put(parameter, wert);
	}

	/**
	 * setzt das Passwort für die Anmeldung beim Datenverteiler bei der
	 * Ausführung des zugeordneten Skripts.
	 *
	 * @param passwd
	 *            das Passwort
	 * @see #getPasswd()
	 */
	public void setPasswd(final String passwd) {
		this.passwd = passwd;
	}

	/**
	 * setzt das zugeordnete Skript des Skriptlaufs.
	 *
	 * @param skript
	 *            das Skript
	 * @see #getSkript()
	 */
	public void setSkript(final Skript skript) {
		this.skript = skript;
	}

	/**
	 * setzt den Starttyp für das Skript.
	 *
	 * @param typ
	 *            der Starttyp
	 * @see #getStartTyp()
	 */
	public void setStartTyp(final StartTyp typ) {
		startTyp = typ;
	}

	/**
	 * setzt den Ausführungsstatus des Skripts.
	 *
	 * @param status
	 *            der Status
	 * @see #getStatus()
	 */
	public void setStatus(final SkriptLaufStatus status) {
		this.status = status;
	}

	/**
	 * setzt den Startzyklus für die Ausführung des Skriptlaufs.
	 *
	 * @param zyklus
	 *            der Zyklus
	 * @see #getZyklus()
	 */
	public void setZyklus(final StartZyklus zyklus) {
		this.zyklus.setStartZeitPunkt(zyklus.getStartZeitPunkt());
		this.zyklus.setIntervall(zyklus.getIntervall());
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion liefert den Namen des Skriptlaufs.
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return getName();
	}
}
